/*==============================================================================================
       ,   .
==============================================================================================*/

#define F_CPU 14745600UL													/*    */
#define COM_RX_BUFFER_SIZE 256 												/*   256 */
#define COM_TX_BUFFER_SIZE 256												/*   256 */
#define COM1_PROTOCOL MODBUS												/*  COM1-   MODBUS*/

#define TIMER2																/*      2  LED.h */
#define SLAVE_COM1 ON														/*   SLAVE*/

#define SCREAN_DO 1															/*    DO(COILS)*/
#define SCREAN_DI 2															/*    DI(Discrete Input)*/
#define SCREAN_AI 3															/*    AI(Input Register)*/
#define SCREAN_AO 4															/*    AO(Holding Register)*/

#include <util/delay.h>														/*   */
#include <avr/io.h>															/*  / ( )*/
#include <avr/interrupt.h>													/*  */

#include <stdio.h>															/*  -      sprintf*/

// #include "MC12D4R4O.h"													/*   MC12D4R4O*/
// #include "MC12D6R.h"														/*   MC12D6R*/
// #include "MC12D8O.h"														/*   MC12D8O*/
// #include "MC4A7D4R4O.h"													/*   MC4A7D4R4O*/
// #include "MC8D2S2R.h"													/*   MC8D2S2R*/
#include "MC8U8O.h"															/*   MC8U8O*/

#include "LED.h"															/*    */
#include "COM.h"															/*    COM-    DCON  Modbus RTU*/
#include "COM_SLAVE.h"														/*   slave Modbus RTU*/
#include "Key.h"															/*     */

volatile uint16_t timer_s = 0;												/* */

ISR(TIMER0_COMP_vect) {														/*  0*/
	timer_s++;																/*    1  timer_s*/
}

void Timer0_Enable(void)													/*    0*/
{
	OCR0 = F_CPU*0.001/1024+0.5;											//  1 
	TIMSK |=(1<<OCIE0);														//     0 
	TCCR0 =(1<<WGM01)|(1<<CS02)|(1<<CS00);									/*  0   "  ".*/
	sei();																	//  
}

int main(void){
	_delay_ms(1000);														/*   */
	LED_Init();																/* */
	Timer0_Enable();														/*  0*/

	uint8_t current_screan = 0;												/*      */
	char message[32];														/*       */
	char title[32];															/*     */
	
	uint8_t test_discrete_data [4] = {1,1,0,0};								/*  */
	uint16_t test_analog_data [2] = {777,877};								/*  */

	uint8_t coils[4];														/*       discrete output*/
	uint8_t DI[4];															/*       discrete input*/
	uint16_t AI[2];															/*       Input Register*/
	uint16_t AO[2];															/*       Holding Register*/

	COM_Init(COM_PORT1, 9600);												/* COM1 */


	Write_multi_COIL(COM_PORT1, test_discrete_data, 4,0);					/*    discrete output*/
	Write_multi_DISCRETE_INPUT(COM_PORT1, test_discrete_data, 4,0);			/*    discrete input*/
	Write_multi_INPUT_REGISTER(COM_PORT1, test_analog_data, 2,0);			/*    Input Register*/
	Write_multi_HOLDING_REGISTER(COM_PORT1, test_analog_data, 2,0);			/*    Holding Register*/
	
	while(1)
	{
		Check_Slave(COM_PORT1);												/*     COM_PORT1*/
		
		switch (current_screan){											/* */
			case SCREAN_DO:													/* discrete output*/
			if (timer_s  > 250){											/*  250   */
				memset(title,0,sizeof(title));								/*  */
				memset(message,0,sizeof(message));							/*  */
				for (int i = 0; i < 4; i++){
					Read_COIL(COM_PORT1, i , &coils[i]);					/* 4    */
				}
				sprintf(message, "%u%u%u%u",								/* */
				coils[0],
				coils[1],
				coils[2],
				coils[3]);
				
				sprintf(title, "SCREAN_DO");								/*  */
			
				LED_Clear();												/* */
				LED_SetPos(0,0);											/*   */
				LED_Write_String(title);									/*    */
				LED_SetPos(1,0);											/*   */
				LED_Write_String(message);									/*   */
				timer_s = 0; 												/* */
			}
			break;															/*( )*/
		
			case SCREAN_DI:													/* discrete input*/
			if (timer_s  > 250){											/*  250   */
				memset(title,0,sizeof(title));								/*  */
				memset(message,0,sizeof(message));							/*  */
				
				for (int i = 0; i < 4; i++){								/*  250   */
					Read_DISCRETE_INPUT(COM_PORT1, i , &DI[i]);				/* 2   discrete input*/
				}	
				sprintf(message, "%u%u%u%u",								/* */
				DI[0],
				DI[1],
				DI[2],
				DI[3]);
				
				sprintf(title, "SCREAN_DI");								/*  */
				LED_Clear();												/* */
				LED_SetPos(0,0);											/*   */
				LED_Write_String(title);									/*    */
				LED_SetPos(1,0);											/*   */
				LED_Write_String(message);									/*   */
				timer_s = 0;												/* */
			}
			break;															/*( )*/
			
			case SCREAN_AI:													/* input register*/

			if (timer_s  > 250){											/*  250   */
				memset(title,0,sizeof(title));								/*  */
				memset(message,0,sizeof(message));							/*  */
				
				for (int i = 0; i < 2; i++){
					Read_INPUT_REGISTER(COM_PORT1, i , &AI[i]);				/* 2   input register*/
				}
				sprintf(message, "%u - %u",									/* */
				AI[0],
				AI[1]);
				
				sprintf(title, "SCREAN_AI");								/*  */
				
				LED_Clear();												/* */
				LED_SetPos(0,0);											/*   */
				LED_Write_String(title);									/*    */
				LED_SetPos(1,0);											/*   */
				LED_Write_String(message);									/*   */
				timer_s = 0;												/* */
			}
			break;															/*( )*/
			
			case SCREAN_AO:													/* holding register*/

			if (timer_s  > 250){											/*  250   */
				memset(title,0,sizeof(title));								/*  */
				memset(message,0,sizeof(message));							/*  */
				
				for (int i = 0; i < 2; i++){
					Read_HOLDING_REGISTER(COM_PORT1, i , &AO[i]);			/* 2   holding register*/
				}
				sprintf(message, "%u - %u",									/* */
				AO[0],
				AO[1]);
				
				sprintf(title, "SCREAN_AO");								/*  */
				
				LED_Clear();												/* */
				LED_SetPos(0,0);											/*   */
				LED_Write_String(title);									/*    */
				LED_SetPos(1,0);											/*   */
				LED_Write_String(message);									/*   */
				timer_s = 0;												/* */
			}
			break;															/*( )*/
			
			default:														/*   */
			if (timer_s  > 250){											/*  250   */
				memset(title,0,sizeof(title));							/*  */
				memset(message,0,sizeof(message));							/*  */
				
				sprintf(title, " ");							/*    */
				sprintf(message, " *>* ");							/*   */
				
				LED_Clear();												/* */
				LED_SetPos(0,0);											/*   */
				LED_Write_String(title);									/*    */
				LED_SetPos(1,0);											/*   */
				LED_Write_String(message);									/*   */
				timer_s = 0;												/* */
			}
		}
		
		if (Key(0) == 1){													/*   1*/
			while (Key(0) == 1){ _delay_ms(10);}							/*   */
			if (current_screan == SCREAN_AO){								/*      */
				current_screan = SCREAN_DO;									/*     */
				}else{
				current_screan ++;											/*   1   */
				timer_s = 255;
			}
		}
		
		if (Key(3) == 1){													/*   1*/
			while (Key(3) == 1){ _delay_ms(10);}							/*   */
			if (current_screan == SCREAN_DO){								/*      */
				current_screan = SCREAN_AO;									/*     */
				}else if(current_screan > 0){
				current_screan --;											/*  1  */
			}
		}
	}
}


